﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Threading;

namespace subsyncer
{
    public partial class Form1 : Form
    {
        string[] lines = null;
        StringBuilder outputBuffer = new StringBuilder();
        Thread timerthread = null;
        MyAppState state = MyAppState.INIT_WAIT;
        List<SubtitleLine> listOfLines;
        string currentLine = "";

        public Form1()
        {
            InitializeComponent();
            timerthread = new Thread(new ThreadStart(run));
        }

        private void button2_Click(object sender, EventArgs e)
        {
            if (openFileDialog1.ShowDialog(this) == DialogResult.OK)
            {
                lines = File.ReadAllLines(openFileDialog1.FileName);
                textBox2.Lines = lines;
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            switch (state)
            {
                case MyAppState.INIT_WAIT:
                    state = MyAppState.IN_LYRICS;
                    timerthread.Start();
                    button1.Text = "Next";
                    break;
                case MyAppState.IN_LYRICS:
                    NextLyricsLine();
                    lock (this)
                    {
                        currentLine = textBox1.Text;
                        if (state != MyAppState.LYRICS_FINISHED)
                        {
                            state = MyAppState.LYRICS_LINE_AVAILABLE;
                        }
                        Monitor.Pulse(this);
                    }
                    break;
                default:
                    break;
            }
        }

        private void NextLyricsLine()
        {
            if (textBox2.Lines.Length > 0)
            {
                textBox1.Text = textBox2.Lines[0];
                string[] dest = new string[textBox2.Lines.Length - 1];
                Array.Copy(textBox2.Lines, 1, dest, 0, dest.Length);
                textBox2.Lines = dest;
            }
            else
            {
                state = MyAppState.LYRICS_FINISHED;
            }
        }

        internal void run()
        {
            DateTime start = DateTime.Now + new TimeSpan(0, 0, 5);
            while (state != MyAppState.STOP_THREAD)
            {
                switch (state)
                {
                    case MyAppState.INIT_WAIT:
                        break;
                    case MyAppState.IN_LYRICS:
                        lock (this)
                        {
                            if (lblTime.InvokeRequired)
                            {
                                TimeSpan ts = DateTime.Now - start;
                                lblTime.BeginInvoke(new EventHandler((sender, e) => lblTime.Text = ts.ToFrenchString()));
                            }
                            Monitor.Pulse(this);
                        }
                        break;

                    case MyAppState.LYRICS_LINE_AVAILABLE: // a new line has been requested by the user
                        TimeSpan ts2 = DateTime.Now - start;
                        if (listOfLines == null) { listOfLines = new List<SubtitleLine>(); }
                        int n = listOfLines.Count;
                        if (n > 0)
                        {
                            listOfLines[n - 1].endTime = ts2; // set the end time of the previous line
                            // TODO: end time should not be set if the previous time was ended manually or via an empty line
                        }

                        SubtitleLine line = new SubtitleLine();
                        line.Number = n + 1; // no protection necessary, object created on this thread.
                        line.startTime = ts2;
                        lock (this)
                        {
                            line.Line = currentLine;
                            state = MyAppState.IN_LYRICS;
                            Monitor.Pulse(this);
                        }
                        listOfLines.Add(line);

                        break;
                    case MyAppState.LYRICS_FINISHED:
                        TimeSpan ts3 = DateTime.Now - start;
                        int n2 = listOfLines.Count;
                        if (n2 > 0)
                        {
                            listOfLines[n2 - 1].endTime = ts3; // set the end time of the previous line
                            // TODO: end time should not be set if the previous time was ended manually or via an empty line
                        }

                        // write it out to a file.
                        StringBuilder sb = new StringBuilder();
                        for (int i = 0; i < n2; i++)
                        {
                            sb.Append((i + 1));
                            sb.Append("\r\n");
                            sb.Append(listOfLines[i].startTime.ToFrenchString());
                            sb.Append(" --> ");
                            sb.Append(listOfLines[i].endTime.ToFrenchString());
                            sb.Append("\r\n");
                            sb.Append(listOfLines[i].Line);
                            sb.Append("\r\n\r\n");
                        }
                        File.WriteAllText("a.srt", sb.ToString());

                        textBox2.BeginInvoke(new EventHandler((sender, e) => textBox2.Text = "Lyrics written to file"));
                        state = MyAppState.STOP_THREAD;

                        break;

                    default:
                        break;
                }

                Thread.Sleep(10);
            }
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            lock (this)
            {
                state = MyAppState.STOP_THREAD;
                Monitor.Pulse(this);
            }
        }

    }
}
